import logging

from spread_core.bam.entities import Entity, Funit
from spread_core.errors.project_errors import CommandError
from spread_core.mqtt.spread import address
from spread_core.mqtt.spread.topic import State
from spread_core.mqtt.spread.variable import Variable


class Subginery(Entity):
    def __init__(self, project_id, data):
        super().__init__(data['id'])
        self._project_id = project_id
        self._location_id = data['locationID']
        self._engineries = dict.fromkeys(data['engineries'])
        self.adapter = None

    def set_adapter(self, adapter):
        self.adapter = adapter

    def publish_state(self, funit_type, value, delay=0):
        if self.get_value(funit_type) != value:
            self.set_value(funit_type, value)
            self.adapter.publish(State(self.get_address(funit_type)), Variable(value), retain=True, delay=delay)

    def get_address(self, funit_type):
        return address.SubgineryAddress(self.project_id, self.id, self.__class__.__name__, funit_type)

    @classmethod
    def get_address_broadcast(cls, p_id, funit_type='+'):
        return address.SubgineryAddress(p_id, '+', cls.__name__, funit_type)

    @property
    def project_id(self):
        return self._project_id

    @property
    def engineries(self): return self._engineries.copy()

    @property
    def location_id(self): return self._location_id

    def has_enginery(self, e_id): return e_id in self._engineries
    def set_enginery(self, enginery): self._engineries[enginery.id] = enginery


class _SubgineriesFunit(Funit):
    entity_type = Subginery
    target_command = None
    target_class = None

    def set(self):
        for e in self.entity.engineries.values():
            if isinstance(e, self.target_class):
                try:
                    e.publish_provider_command(self.target_command, self.set_value, self.params, sig=self.signature)
                except CommandError as ex:
                    logging.warning(ex)
