from . import Topic


class ProjectTopic(Topic):
    protocol = 'Project'

    def __init__(self, project_id: int):
        self.project_id = project_id

    def __str__(self): return '/'.join((self.protocol, str(self.project_id), '#'))


class ProjectHash(ProjectTopic):
    HASH = 'Hash'

    def __str__(self): return '/'.join((self.protocol, str(self.project_id), self.HASH))


class TopicProject(ProjectTopic):
    ENGINERIES = 'engineries'
    PROVIDERS = 'providers'
    MANAGERS = 'managers'
    SUBGINERIES = 'subgineries'
    HEADER = 'header'
    SERVERS = 'servers'
    LOCATIONS = 'locations'
    MODELS = 'models'
    CTP = 'Ctp'

    topic_type = 'File'

    def __init__(self, project_id, file_name):
        super(TopicProject, self).__init__(project_id)
        self.file_name = file_name

    def __str__(self):
        if self.file_name == TopicProject.CTP:
            return f'{self.protocol}/{self.project_id}/{self.file_name}'
        return f'{self.protocol}/{self.project_id}/{self.topic_type}/{self.file_name}'

    @classmethod
    def engineries(cls, p_id): return cls(p_id, f'{TopicProject.ENGINERIES}.json')

    @classmethod
    def subgineries(cls, p_id): return cls(p_id, f'{TopicProject.SUBGINERIES}.json')

    @classmethod
    def providers(cls, p_id): return cls(p_id, f'{TopicProject.PROVIDERS}.json')

    @classmethod
    def managers(cls, p_id): return cls(p_id, f'{TopicProject.MANAGERS}.json')

    @classmethod
    def servers(cls, p_id): return cls(p_id, f'{TopicProject.SERVERS}.json')

    @classmethod
    def header(cls, p_id): return cls(p_id, f'{TopicProject.HEADER}.json')

    @classmethod
    def locations(cls, p_id): return cls(p_id, f'{TopicProject.LOCATIONS}.json')

    @classmethod
    def ctp(cls, p_id): return cls(p_id, TopicProject.CTP)
